<?php
/**
 * Copyright since 2022 Inferendo Srl
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to hello@visidea.ai so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade Visidea to newer
 * versions in the future. If you wish to customize Visidea for your
 * needs please refer to https://visidea.ai/ for more information.
 *
 * @author    Inferendo SRL <hello@visidea.ai>
 * @copyright Since 2022 Inferendo Srl
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

class visideaCronModuleFrontController extends ModuleFrontController
{
    public function initContent()
    {
        parent::initContent();

        $this->generateCsv();

    }

    private function generateCsv()
    {
        $cronKey = Configuration::get('VISIDEA_PRIVATE_KEY');

        $urlVar = $_GET;
        if (!isset($urlVar['token']) || !$urlVar['token'] || $cronKey != $urlVar['token'])
        {
            return false;
        }

        $path = __DIR__ . "/../../csv/";
        if (!is_dir($path)) {
            mkdir($path, 0777, true);
        }


        // file items
        $items = $this->getItems();
        $filename = 'items_' . $urlVar['token'] . '.csv';

        if (file_exists($path . $filename)) {
            unlink($path . $filename);
        }


        $output = fopen($path . $filename, "w");
        $header = array_keys($items[0]);
        fputcsv($output, $header, ";");

        foreach ($items as $row) {
            fputcsv($output, $row, ";");
        }
        fclose($output);


        // file users
        $users = $this->getUsers();
        $filename = 'users_' . $urlVar['token'] . '.csv';

        if (file_exists($path . $filename)) {
            unlink($path . $filename);
        }

        $output = fopen($path . $filename, "w");
        $header = array_keys($users[0]);
        fputcsv($output, $header, ";");

        foreach ($users as $row) {
            fputcsv($output, $row, ";");
        }
        fclose($output);


        // file interactions
        $cartProducts = $this->getCarts();
        $filename = 'interactions_' . $urlVar['token'] . '.csv';

        if (file_exists($path . $filename)) {
            unlink($path . $filename);
        }

        $output = fopen($path . $filename, "w");
        $header = array_keys($cartProducts[0]);
        fputcsv($output, $header, ";");

        foreach ($cartProducts as $row) {
            fputcsv($output, $row, ";");
        }
        fclose($output);

        die();
    }


    private function getItems()
    {
        $id_lang = (int)Configuration::get('PS_LANG_DEFAULT');

        $sql = 'SELECT `id_product` FROM `' . _DB_PREFIX_ . 'product`';
        $productsId = Db::getInstance()->executeS($sql);

        $return = array();

        foreach ($productsId as $productId)
        {
            $newProduct = array();
            $newProduct['item_id'] = $productId['id_product'];

            $prod = new Product($newProduct['item_id']);

            $newProduct['name'] = $prod->name[$id_lang];
            $newProduct['brand_id'] = $prod->id_manufacturer;
            $newProduct['brand_name'] = Manufacturer::getNameById((int)$newProduct['brand_id']);
            $newProduct['price'] = $prod->getPriceWithoutReduct();
            $newProduct['market_price'] = $prod->getPrice();
            $newProduct['discount'] = abs(($newProduct['market_price'] - $newProduct['price']) / $newProduct['price'] * 100);

            $categories = Product::getProductCategoriesFull($newProduct['item_id']);
            $catIds = array();
            $catNames = array();
            foreach ($categories as $cat)
            {
                $catIds[] = $cat['id_category'];
                $catNames[] = $cat['name'];
            }
            $newProduct['page_ids'] = implode('|', $catIds);
            $newProduct['page_names'] = implode('|', $catNames);

            $link = new Link();
            $newProduct['url'] = $link->getProductLink($prod);

            $productImages = $prod->getImages((int)$id_lang);
            $imagePathArr = array();
            foreach ($productImages AS $key => $val)
            {
                $id_image = $val['id_image'];
                $imagePath = "http://".$link->getImageLink($prod->link_rewrite[Context::getContext()
                    ->language->id], $id_image, 'home_default');
                $imagePathArr[] = $imagePath;
            }
            $newProduct['images'] = implode('|', $imagePathArr);

            $newProduct['stock'] = Product::getQuantity($newProduct['item_id']);

            $return[] = $newProduct;
        }

        return $return;
    }

    private function getUsers()
    {
        $sql = 'SELECT `id_customer` FROM `' . _DB_PREFIX_ . 'customer`';
        $customersId = Db::getInstance()->executeS($sql);

        $return = array();

        foreach ($customersId as $customerId)
        {
            $newCustomer = array();
            $newCustomer['user_id'] = $customerId['id_customer'];

            $cust = new Customer((int)$newCustomer['user_id']);

            $newCustomer['email'] = $cust->email;
            $newCustomer['name'] = $cust->firstname;
            $newCustomer['surname'] = $cust->lastname;

            $addressId = Address::getFirstCustomerAddressId($newCustomer['user_id']);
            $address = new Address($addressId);

            $newCustomer['address'] = $address->address1;
            $newCustomer['city'] = $address->city;
            $newCustomer['zip'] = $address->postcode;

            $sql = 'SELECT `name` FROM `' . _DB_PREFIX_ . 'state` WHERE `id_state` = ' . $address->id_state . ' AND `id_country` = ' . $address->id_country;
            $state = Db::getInstance()->getRow($sql);
            $newCustomer['state'] = $state['name'];

            $sql = 'SELECT `iso_code` FROM `' . _DB_PREFIX_ . 'country` WHERE `id_country` = ' . $address->id_country;
            $country = Db::getInstance()->getRow($sql);
            $newCustomer['country'] = strtolower($country['iso_code']);

            $newCustomer['birthday'] = $cust->birthday;

            $return[] = $newCustomer;
        }

        return $return;
    }

    private function getCarts()
    {
        $sql = 'SELECT * FROM `' . _DB_PREFIX_ . 'cart_product` INNER JOIN ' . _DB_PREFIX_ . 'cart ON ' . _DB_PREFIX_ . 'cart_product.id_cart=' . _DB_PREFIX_ . 'cart.id_cart';
        $result = Db::getInstance()->executeS($sql);

        $oSql = 'SELECT `id_order`,`id_cart` FROM `' . _DB_PREFIX_ . 'orders`';
        $orderList = Db::getInstance()->executeS($oSql);

        $oItm = array();
        foreach ($orderList as $oderItm)
        {
            $oItm[$oderItm['id_order']] = $oderItm['id_cart'];
        }

        $return = array();
        $cartProduct = array();
        foreach ($result as $product)
        {
            if ($product['id_customer']) $cartProduct['user_id'] = $product['id_customer'];
            else $cartProduct['user_id'] = $product['id_guest'];
            $cartProduct['item_id'] = $product['id_product'];
            if (in_array($product['id_cart'], $oItm)) $cartProduct['action'] = 'purchase';
            else $cartProduct['action'] = 'cart';

            $timestamp = new DateTime($product['date_upd']);
            $cartProduct['timestamp'] = $timestamp->format(DateTime::ISO8601);

            $return[] = $cartProduct;
        }

        return $return;
    }

}
